//! Position.cpp

#include <omnetpp.h>
#include <assert.h>
#include "Node.h"

#define MIN_DIST 50

#define MAX_FAIL 20
#define FAIL_MIN_DIST_DECREASE 5
#define FAIL_MIN_DIST 25

//! Position simple module class definition
/*
    This module configures the network topology.
    Its main purpose is to position nodes in the network.

	\author	Nuno Pereira
	\date May 2006.
*/
class Position : public cSimpleModule
{
// Member variables
private:
	// retrieved module parameters and pre-calculated data
	int MIN_PX;
	int MIN_PY;
	int MAX_PX;
	int MAX_PY;
			
	// internal, calculated variables
	int MINIMUM_DISTANCE;
	
// Member functions
public:
    Module_Class_Members(Position,cSimpleModule,0)
	//! Do initialization of class data members and any other initialization procedure necessary
    virtual void initialize();
	//! Process messages received.
    virtual void handleMessage(cMessage *msg);
	//! Saves summary results (if any)
    virtual void finish();

	virtual ~Position();
	
// Aditional methods
private:

};

Define_Module( Position );

//!Do initialization of class data members and any other initialization procedure necessary
void Position::initialize()
{
	MIN_PX = (int) par("min_px");
	MIN_PY = (int) par("min_py");
	MAX_PX = (int) par("max_px");
	MAX_PY = (int) par("max_py");
	
	MINIMUM_DISTANCE = MIN_DIST;
	
		
	// get system module
	cModule *sysModule = parentModule();
	while (sysModule->parentModule() != NULL) sysModule = sysModule->parentModule();
	ASSERT(sysModule != NULL);

	// find all nodes in the network and set their position 
	for (cSubModIterator modIter(*sysModule); !modIter.end(); modIter++)
	{
		cModule *module = modIter();
		if (dynamic_cast<Node *>(module) != NULL) {

			cModule *ourNode = (Node *) module;
			bool pos_ok = false;
			int fail_count = 0;
			
			while (!pos_ok) {
				// position of the node
				double our_px = uniform(MIN_PX, MAX_PX);
				double our_py = uniform(MIN_PY, MAX_PY);
				
				pos_ok = true;
				
				// find all other nodes in the network  
				for (cSubModIterator mIt(*sysModule); !mIt.end(); mIt++)
				{
					module = mIt();
					if (dynamic_cast<Node *>(module) != NULL) {
						Node *node = (Node *) module;
						if (node != ourNode) {
							double px = (double) node->par("px");
							double py = (double) node->par("py");
							double dx = our_px - px;
							double dy = our_py - py;
							double dist = sqrt(dx*dx + dy*dy);
							if (dist < MINIMUM_DISTANCE) {
								pos_ok = false;
								fail_count++;
								if (fail_count == MAX_FAIL) {
									fail_count = 0;
									MINIMUM_DISTANCE-=FAIL_MIN_DIST_DECREASE;
									assert (FAIL_MIN_DIST < MINIMUM_DISTANCE); // This means we have no space to put the nodes: Increace MAX_PX and MAX_PY.
								} 
								break;
							}
						}
					}
				}
				ourNode->par("px") = our_px;
				ourNode->par("py") = our_py;
				char dispStr[50];
				sprintf(dispStr, "p=%f,%f;i=misc/node_vs", our_px, our_py);
				ourNode->setDisplayString(dispStr, true);
			}
		}
	}
	std::cout << "\n";
}

//!	Process messages received.
/*!

	\param msg - cMessage* : Message received.
*/
void Position::handleMessage(cMessage *msg)
{
}

//! Saves summary results (if any)
void Position::finish()
{
    // save summary results (if any)
}

Position::~Position()
{
}
