//! Stream.cpp

#include <omnetpp.h>

#include "MessageTypes.h"

//! DataGen simple module class definition
/*
    Models a data generator.
	Sets the timestamp of the messages, so the receiving output can calculate
	the end-to-end response time.

	\author	Nuno Pereira
	\date Oct 2005.
*/
class Stream : public cSimpleModule
{
// Member variables
private:
	// retrieved module parameters and pre-calculated data

	//! retrieved module parameter; Size of the generated data
	cPar*		dataLen;				
	//! retrieved module parameter; Period of generation
	cPar*		t;				
	//! retrieved module parameter; Priority of generated messages
	int			prio;				

	// internal, calculated variables
	int			sequenceNumber;		//! sequence number for the data sent

// Member functions
public:
    Module_Class_Members(Stream,cSimpleModule,0)
	//! Do initialization of class data members and any other initialization procedure necessary
    virtual void initialize();
	//! Process messages received.
    virtual void handleMessage(cMessage *msg);
	//! Saves summary results (if any)
    virtual void finish();

	virtual ~Stream();
	
// Aditional methods
private:
	//! Generates and sends a data item.
	void sendDataGen();
};

Define_Module( Stream );

//!Do initialization of class data members and any other initialization procedure necessary
void Stream::initialize()
{
	// get module parameters;

	// these are taken by reference because they can be a random function
	dataLen = &par("dataLen");
	t = &par("t");
	prio = (int) par("prio");

    // initialize class data members
    sequenceNumber = 1;

	// schedule first generation message
	if ((t->type() == 'D' || t->type() == 'L') && ((double)*t) == 0) return;  // period is zero; this generator is disabled
	
	cMessage *msg = new cMessage("MSG_STREAM_DATA_MSG");
	double actTime = ((double) par("actTime"));
	// CHANGE: TEMP
	actTime=0;
  scheduleAt(actTime, msg);
	////////
	//if (actTime >= 0) scheduleAt(actTime, msg);
}

//!	Process messages received.
/*!
	This will be called only when receiving a generateNextDataGen message.
	Generates a data item message and schedules next generation.

	\param msg - cMessage* : Message received.
*/
void Stream::handleMessage(cMessage *msg)
{
	sendDataGen();
	scheduleAt(simTime() + ((double)*t), msg);
}

//! Generates and sends a data item.
/*!
	The stream data item is sent here
	The timestamp is set with current time, so the receiving output can calculate
	the end-to-end response time of this message.
*/
void Stream::sendDataGen()
{
	cMessage *msg = new cMessage("MSG_STREAM_DATA_MSG");

	msg->setLength(((long)*dataLen)); // Data length (bytes)
	msg->setTimestamp();
	//msg->setPriority(prio);
	msg->setPriority(parentModule()->index());
	msg->setKind(MSG_TYPE_DATA_MSG);
	// send message
	send(msg, "out");
}

//! Saves summary results (if any)
void Stream::finish()
{
    // save summary results (if any)
}

Stream::~Stream()
{	
}
